--  
--  $Header: ch1_drv.ada,v 1.2 1994/04/22 05:40:18 cracicot Exp $ 
--  
-- Copyright (c) 1992, 1999 by Oracle Corporation 
--   NAME
--     ch1_drv.ada - <one-line expansion of the name>
--   DESCRIPTION
--     <short description of component this file declares/defines>
--   RETURNS
--
--   NOTES
--     <other useful comments, qualifications, etc.>
--   MODIFIED   (MM/DD/YY)
--    akatti     04/15/99 -  dont use service_name while connecting
--    tssmith    02/20/94 -  Branch_for_patch
--    tssmith    02/20/94 -  Creation
-- Include TEXT_IO,SQL_STANDARD and CH1_MOD package specs.

with
     SQL_STANDARD,
     TEXT_IO,
     CH1_MOD;
use
     SQL_STANDARD;


-- Define the main procedure.

procedure CH1_DRV is


-- Instantiate new packages for I/O on SQL_STANDARD datatypes.
    package STD_INT_IO is
        new text_io.integer_io(SQL_STANDARD.INT);
    use STD_INT_IO;

    package SQLCODE_IO is
        new text_io.integer_io(SQL_STANDARD.SQLCODE_TYPE);
    use SQLCODE_IO;

    package STD_FLOAT_IO is
        new text_io.float_io(SQL_STANDARD.REAL);
    use STD_FLOAT_IO;


-- Begin with declarations of all program variables, 
-- including parameters for the module procedures.
    USERNAME             : SQL_STANDARD.CHAR(1..12)
                               := "SCOTT       ";
    PASSWORD             : SQL_STANDARD.CHAR(1..12)
                               := "TIGER       ";

    DEPT_NUMBER          : SQL_STANDARD.INT;
    EMPLOYEE_NUMBER      : SQL_STANDARD.INT; 
    EMPLOYEE_NAME        : SQL_STANDARD.CHAR(1..10); 
    SALARY               : SQL_STANDARD.REAL; 
    COMMISSION           : SQL_STANDARD.REAL;
    COMM_IND             : SQL_STANDARD.SMALLINT;
    SQLCODE              : SQL_STANDARD.SQLCODE_TYPE; 
    LENGTH               : integer;

    CONNECT_ERROR        : exception;
    SQLCODE_ERROR        : exception; 
    
begin 

-- Call a module procedure to connect
-- to the Oracle server.
    CH1_MOD.DO_CONNECT
        (USERNAME, PASSWORD, SQLCODE);

-- Test SQLCODE to see if the connect succeeded.
    if SQLCODE /= 0 then
        raise CONNECT_ERROR;
    end if;

    TEXT_IO.NEW_LINE(2);

-- Get an input value from user to use
-- in opening the cursor.
    TEXT_IO.PUT("Enter department number: ");
    GET(DEPT_NUMBER);
    TEXT_IO.NEW_LINE;

-- Call the module procedure to open the cursor.
-- You open the cursor using the input parameter (dept_number).
    CH1_MOD.OPEN_CURSOR1(DEPT_NUMBER, SQLCODE); 

-- If SQLCODE indicates error, call exception handler.
    if SQLCODE < 0 then
        raise SQLCODE_ERROR;
    end if;

    TEXT_IO.PUT_LINE("Employee  ID Number   Salary  Commission");
    TEXT_IO.PUT("----------------------------------------");

-- Call the FETCH procedure in a loop, to get
-- the employee data.
    loop
        CH1_MOD.FETCH_EMP_DATA
               (EMPLOYEE_NUMBER, 
                EMPLOYEE_NAME, 
                SALARY, 
                COMMISSION,
                COMM_IND,
                SQLCODE);

        TEXT_IO.NEW_LINE;    

-- When SQLCODE = 100, no more rows to fetch.
        exit when SQLCODE = 100;

-- When SQLCODE less than 0, an error occurred.
        if SQLCODE < 0 then
            raise SQLCODE_ERROR;
        end if;

        TEXT_IO.PUT(string(EMPLOYEE_NAME));
        PUT(EMPLOYEE_NUMBER, WIDTH => 9); 
        PUT(SALARY, FORE => 6, AFT => 2, EXP => 0); 

        if COMM_IND = 0 then
            PUT(COMMISSION, FORE => 9, AFT => 2, EXP => 0);
        else
            TEXT_IO.PUT("        Null");
        end if;
    end loop;

    TEXT_IO.NEW_LINE(2);

-- Call the procedure to close the cursor.
    CH1_MOD.CLOSE_CURSOR1(SQLCODE);

-- Call the procedure to disconnect.
    CH1_MOD.DO_DISCONNECT(SQLCODE);

-- Handle the error exception.

exception 
     when SQLCODE_ERROR => 
          TEXT_IO.NEW_LINE(2);
          TEXT_IO.PUT("Error fetching data, SQLCODE returns "); 
          PUT(SQLCODE); 
          TEXT_IO.NEW_LINE(2);

     when CONNECT_ERROR =>
          TEXT_IO.PUT("Connect failure ");
          TEXT_IO.NEW_LINE(2);

end CH1_DRV; 

