include $(ORACLE_HOME)/precomp/lib/env_precomp.mk

# This Makefile builds the sample programs in
# $(ORACLE_HOME)/precomp/demo/modada), and can serve as a template for
# linking customer applications.

# $(SAMPLES) is a list of the modada sample programs.
SAMPLES=demcalsp demohost ch1_drv

# These targets build all the sample programs in one call to make.
# 'all' creates the user for the sample programs and loads some data into
# tables, first.
# 'samples' assumes that the database is mostly setup already.
all: makeuser loaddb $(SAMPLES)
samples: $(SAMPLES)

# The samples require that .sql scripts be run before precompilation.
# If you set RUNSQL=run in your call to make, then make will use sqlplus or
# svrmgrl, as appropriate, to run the .sql scripts.
# If you leave RUNSQL unset, then make will print out a reminder to run the
# scripts.
# If you have already run the scripts, then RUNSQL=done will omit the reminder.
sqlplus_run:
	($(CD) ../sql; $(BINHOME)sqlplus $(USER) @$(SCRIPT) < /dev/null)
svrmgrl_run:
	($(CD) ../sql; $(BINHOME)svrmgrl < $(SCRIPT).sql)
sqlplus_ svrmgrl_:
	$(SILENT)$(ECHO) "# You must run the .sql script, " $(SCRIPT),
	$(SILENT)$(ECHO) "# before precomping this sample."
sqlplus_done svrmgrl_done:
#
gpapkg-sql:
	$(MAKE) -f $(MAKEFILE) USER=$(USERID) SCRIPT=gpapkg sqlplus_$(RUNSQL)

# The target makeuser creates the user in the database for these samples.
# It requires the system password.
makeuser:
	$(MAKE) -f $(MAKEFILE) USER=system/$(SYSTEMPASS) SCRIPT=grant sqlplus_$(RUNSQL)

# The target loaddb creates tables for some of the sample programs.
loaddb:
	$(MAKE) -f $(MAKEFILE) USER=$(USERID) SCRIPT=mktables sqlplus_$(RUNSQL)

# These targets make the specific sample programs:
demohost:
	$(MAKE) -f $(MAKEFILE) ada OBJS="demomods.ada demomod.ada demohost.ada" \
	EXE=demohost MODARGS=user=$(USERID)

demcalsp: 
	$(MAKE) -f $(MAKEFILE) ada OBJS="demomod.ada demcalsp.ada gpa_pkg.ada" \
	EXE=demcalsp MODARGS=user=$(USERID)

ch1_drv:
	make -f $(MAKEFILE) ada OBJS="ch1_mod.ada ch1_drv.ada" EXE=ch1_drv \
	MODARGS=user=$(USERID1)

# These are the rules for converting .mad -> .a or .ada.
.SUFFIXES: .mad .a .ada

.mad.ada: env_$(ADAPORT)
	$(BINHOME)/modada $(MODARGS) iname=$*.mad
	$(CP) $*.a $*.ada
	$(CP) $*s.a $*s.ada
	$(ADACC) $*s.ada 

ada: ada_$(ADAPORT)

# gpapkg.ada is created by modada using tables in the database as source, 
# rather than a .mad file.
gpa_pkg.ada: gpapkg-sql env_$(ADAPORT)
	$(BINHOME)/modada pname=GPA_PKG user=$(USERID) rpc_generate=yes oname=gpa_pkg 
	$(CP) gpa_pkg.a gpa_pkg.ada
	$(CP) gpa_pkgs.a gpa_pkgs.ada
	$(ADACC) gpa_pkgs.ada

# Here are some rules that depend on the ada vendor for the particular 
# platform.
ada_verdix: env_$(ADAPORT) $(OBJS)
	$(ADACC) $(OBJS) 
	$(ADALD) $(EXE) -o $(EXE) $(LDPATHFLAG)$(LIBHOME) $(PROLDLIBS)

env_verdix: GVAS_table_verdix
GVAS_table_verdix:
	a.rmlib -f
	-echo 1 | a.mklib -i > /dev/null ; a.path -i $(PRECOMPADALIB)

env_alsys: demo_alsys
demo_alsys:
	create_lib demo_alsys

env_dec: adalib 
adalib:
	amklib -p@$(PRECOMPADALIB)/adalib adalib

# These macro definitions fill in some details or override some defaults
MAKEFILE=$(ORACLE_HOME)/precomp/demo/modada/demo_modada.mk
SYSTEMPASS=manager
USERID=modtest/yes
USERID1=scott/tiger
