# This Makefile builds the sample programs in 
# $(ORACLE_HOME)/precomp/demo/profor, and can serve as a template for linking
# customer applications.

include $(ORACLE_HOME)/precomp/lib/env_precomp.mk

# SAMPLES is a list of the fortran sample programs.
SAMPLES=sample1 sample2 sample3 sample4 sample6 sample7 sample8 \
	sample9 sample10 sample11

# This target builds all the samples in one call to make.
samples: clean $(SAMPLES)

# The target 'build' puts together an executable from the fortran sources
# in $(FORS) and the libraries in $(PROLDLIBS).
# The rules to make .f files from .pfo files are later in this file.
# The target 'build_static' is similar, but does not use the client shared
# library.
build: $(FORS)
	$(DEMO_PROFOR_BUILD_SHARED)
build_static: $(FORS)
	$(DEMO_PROFOR_BUILD_STATIC)

$(SAMPLES):
	$(MAKE) -f $(MAKEFILE) build FORS=$@.f EXE=$@

#-------------------------------------------------------------------
# Clean up all executables, *.o and generated *.for or *.f files
clean:
	$(SILENT)$(RMF) $(SAMPLES)
	$(SILENT)$(RMF) sample*.f sample*.for SAMPLE*.F SAMPLE*.FOR
	$(SILENT)$(RMF) sample*.lis SAMPLE*.LIS
	$(SILENT)$(RMF) sample*.$(OBJ_EXT)

# sample5.pfo illustrates the precompiler part of building a precompiler-forms
# application.  Building the actual executable would require the FORMS bundle.
sample5:
	$(SILENT)$(ECHO) 'sample5 is a user-exit demo; use a makefile from forms.'

# Some of the samples require that .sql scripts be run before precompilation.
# If you set RUNSQL=run in your call to make, then make will use sqlplus or
# svrmgrl, as appropriate, to run the .sql scripts.
# If you leave RUNSQL unset, then make will print out a reminder to run the
# scripts.
# If you have already run the scripts, then RUNSQL=done will omit the reminder.
sqlplus_run:
	($(CD) ../sql; $(BINHOME)sqlplus $(USER) @$(SCRIPT) < /dev/null)
svrmgrl_run:
	($(CD) ../sql; $(BINHOME)svrmgrl < $(SCRIPT).sql)
sqlplus_ svrmgrl_:
	$(SILENT)$(ECHO) "# You must run the .sql script, " $(SCRIPT),
	$(SILENT)$(ECHO) "# before precomping this sample."
sqlplus_done svrmgrl_done:
#
calldemo-sql:
	$(MAKE) -f $(MAKEFILE) USER=$(USERID) SCRIPT=calldemo sqlplus_$(RUNSQL)
sample11-sql:
	$(MAKE) -f $(MAKEFILE) USER=$(USERID) SCRIPT=sample11 sqlplus_$(RUNSQL)

# Here are some rules for converting .pfo -> ..f -> .o.
.SUFFIXES: .f .pfo .o

.pfo.f:
	$(PROFOR) $(PCCFLAGS) iname=$*.pfo

.f.o:
	$(F2O)

# These are specific targets to make the .o files for samples that require
# more careful handling.
sample9.f: calldemo-sql
	$(PROFOR) $(PCCPLSFLAGS) iname=$*.pfo dbms=v6

sample10.f: BNDDSC.FOR SELDSC.FOR
	$(PROFOR) $(PCCFLAGS) iname=$*.pfo dbms=v6

sample11.f: sample11-sql
	$(PROFOR) $(PCCPLSFLAGS) iname=$*.pfo dbms=v6

BNDDSC.FOR: $(PRECOMPHOME)public/bnddsc.for
	$(RMF) BNDDSC.FOR
	$(LNS) $(PRECOMPHOME)public/bnddsc.for BNDDSC.FOR
SELDSC.FOR: $(PRECOMPHOME)public/seldsc.for
	$(RMF) SELDSC.FOR
	$(LNS) $(PRECOMPHOME)public/seldsc.for SELDSC.FOR

# These macro definitions fill in some details or override some defaults
MAKEFILE=$(ORACLE_HOME)/precomp/demo/profor/demo_profor.mk
PCCPLSFLAGS=sqlcheck=full userid=$(USERID)
USERID=scott/tiger
